import 'package:redstone/server.dart' as app;
import 'package:rethinkdb_driver/rethinkdb_driver.dart';
import 'package:rethinkdb_driver/rethinkdb_pool.dart';
import 'package:redstone_mapper/database.dart';
import 'package:connection_pool/connection_pool.dart';
import 'dart:async';

export 'package:connection_pool/connection_pool.dart';

class RethinkDbManager extends DatabaseManager<ManagedConnection<Connection>>{

  Rdb _pool;

  RethinkDbManager(String host, String database, {int port: 28015, String auth: "", int poolSize: 3}) {
    _pool = new Rdb(database, port, host, auth, poolSize);
  }

  @override
  void closeConnection(RethinkConnection connection, {error}) {
    var invalidConn = error is RqlDriverError;
    _pool.releaseConnection(
        connection,
        markAsInvalid: invalidConn);
  }

  @override
  Future<RethinkConnection> getConnection() async {
    ManagedConnection<Connection> mConn = await _pool.getConnection();
    return new RethinkConnection(mConn.connId, mConn.conn);
  }
}

class RethinkConnection extends ManagedConnection<Connection> implements Connection{

  RethinkConnection(int connId, Connection conn) : super (connId, conn);
  
  @override
  void addListener(String key, Function val) {
    conn.addListener(key, val);
  }

  @override
  close([bool noreply_wait = true]) {
    conn.close(noreply_wait);
  }

  @override
  Future<Connection> connect([bool noreply_wait = true])
    => conn.connect(noreply_wait);

  // TODO: implement isClosed
  @override
  get isClosed => conn.isClosed;

  @override
  noreplyWait() {
    conn.noreplyWait();
  }

  @override
  void on(String key, Function val) {
    conn.on(key, val);
  }

  @override
  use(String db) {
    conn.use(db);
  }
}